.nolist
.include "m8def.inc"
.include "nrf24l01.inc"
.list

.cseg
.org 0000
rjmp init ;reset 
rjmp isr0 ;int0
reti ;int1
reti ;timer2 comp
reti ;timer2 overflow
reti ;timer1 capture
reti ;timer1 compare A
reti; timer1 compare N
reti ;timer1 overflow
reti ;timer0 overflow
reti ;SPI serial transfer complete
reti ;USART rx complete
reti ;USART DR empty
reti ;USART tx complete
reti ;ADC complete
reti ;eeprom ready
reti ;analog comparator
reti ;two wire interface
reti ;store program memory ready


init:
;INIT STACK
ldi r16, HIGH(RAMEND) ; Oberes Byte
out SPH, r16 ; an Stapelzeiger
ldi r16, LOW(RAMEND) ; Unteres Byte
out SPL, r16 ; an Stapelzeiger

;SATURN D0-D3 => PORTC3:0 = OUTPUT
ldi r16, 0b0001_1111
out ddrc, r16

;VARIABLES FOR RECEIVED BUTTONS
.equ ID = SRAM_START
.equ BUTTONS1 = SRAM_START+1
.equ BUTTONS2 = SRAM_START+2

;N64 VARIABLES
.equ COMMAND = SRAM_START+3
.equ N64_BUTTONS1 = SRAM_START+4
.equ N64_BUTTONS2 = SRAM_START+5
.equ N64_X = SRAM_START+6
.equ N64_Y = SRAM_START+7
.equ MODE_SELECT = SRAM_START+8	;FOR SELECT MODE
.equ SELECT_PRESSED = SRAM_START+9
ldi r16, 0x00
sts n64_buttons1, r16
sts n64_buttons2, r16
sts n64_x, r16
sts n64_y, r16
sts mode_select, r16
sts select_pressed, r16


;CONFIG SPI PINS
;SCK output ;MISO input;MOSI output;SS output;CE output;IRQ input
LDI R16,(1<<0)|(1<<2)|(1<<3)|(1<<5)
out DDRB, r16
;Pullups for IRQ + MISO
ldi r16, (1<<4)|(1<<1)
out PORTB, r16

;CONFIG SPI
; Enable SPI, Master, clockrate f/4, MSB, 
ldi r17,(1<<SPE)|(1<<MSTR)
out SPCR,r17


;CONFIG CHANNEL SELECT SWITCH; PULLUP FOR PIND.0
ldi r16, 0b0001_1000
out portd, r16
.equ CHANNEL = SRAM_START+10
rcall channel_select


;CONFIG INT0 = low level
ldi r16, 0b0000_0000
out MCUCR, r16
ldi r16, 0b0100_0000
out GICR, r16
sei

;CONFIG NRF
rcall setup_nrf



main:
;***CHECK IF CHANNEL HAS BEEN CHANGED
check_channel:
	lds r16, channel
	sbis pind,3
	rjmp channel_switch_high

channel_switch_low:
	ldi r17, 109
	sub r17, r16
	BRNE wrong_channel ;FALSCHER KANAL
	RJMP check_channel_end ;RICHTIGER KANAL

channel_switch_high:
	ldi r17, 119
	sub r17, r16
	BRNE wrong_channel
	RJMP check_channel_end

wrong_channel:
	rcall channel_select
	rcall setup_nrf

check_channel_end:


;IF IRQ=1 THEN GOTO MAIN
	in r16, pinb
	sbrc r16, 1
	rjmp main
;IF IRQ=0 THEN:
	RESET_CE
read_fifo:
	RESET_SS
	ldi r17, rd_rx_pload
	rcall do_spi
	rcall do_spi
	sts ID, r17
	rcall do_spi
	sts BUTTONS1, r17
	rcall do_spi
	sts BUTTONS2, r17
		;------------------------
		;NEW
		ANALOG_DATA:
		rcall do_spi
		sts n64_x, r17
		rcall do_spi
		sts n64_y, r17
		ANALOG_DATA_END:
		;------------------------
	SET_SS
	rcall translate_buttons
	rcall wait10ms
;read fifo status
RESET_SS
ldi r17, fifo_status
rcall do_spi
rcall do_spi
SET_SS
sbrs r17, 0
rjmp read_fifo
;IF FIFO=EMPTY THEN CLEAR RX_DR BIT
RESET_SS
ldi r17, write_reg+status
rcall do_spi
ldi r17, 0b01000000
rcall do_spi
SET_SS
SET_CE
rjmp main




isr0:
in r20, sreg ;save status register
rcall delay16 ;wait 16 = 1us
rcall delay16 ;wait 16 = 1us

ldi r18, 0xFF ;reset r18
;***BIT 7
sbis pind, 2 ;if data=1, skip
cbr r18, 0b1000_0000 ;else, clear bit
rcall delay62 ;wait 3,8125us
;***BIT 6
sbis pind, 2 ;if data=1, skip
cbr r18, 0b0100_0000 ;else, clear bit
rcall delay62 ;wait 3,8125us
;***BIT 5
sbis pind, 2 ;if data=1, skip
cbr r18, 0b0010_0000 ;else, clear bit
rcall delay62 ;wait 3,8125us
;***BIT 4
sbis pind, 2 ;if data=1, skip
cbr r18, 0b0001_0000 ;else, clear bit
rcall delay62 ;wait 3,8125us
;***BIT 3
sbis pind, 2 ;if data=1, skip
cbr r18, 0b0000_1000 ;else, clear bit
rcall delay62 ;wait 3,8125us
;***BIT 2
sbis pind, 2 ;if data=1, skip
cbr r18, 0b0000_0100 ;else, clear bit
rcall delay62 ;wait 3,8125us
;***BIT 1
sbis pind, 2 ;if data=1, skip
cbr r18, 0b0000_0010 ;else, clear bit
rcall delay62 ;wait 3,8125us
;***BIT 0
sbis pind, 2 ;if data=1, skip
cbr r18, 0b0000_0001 ;else, clear bit
rcall delay62 ;wait 3,8125us

sts COMMAND, r18
rcall delay16
rcall delay16
rcall delay16
rcall delay16

;***CHECK COMMAND SENT BY N64
ldi r19, 0x00
lds r18, COMMAND
sub r18, r19
breq send_status

ldi r19, 0x01
lds r18, COMMAND
sub r18, r19
breq send_buttons

ldi r19, 0xFF
lds r18, COMMAND
sub r18, r19
breq send_status




isr0_end:
out sreg, r20 ;write back status register
reti




send_status:
ldi r18, 0x05
nop
rcall send_a_byte
ldi r18, 0x00
nop
rcall send_a_byte
ldi r18, 0x02
nop
rcall send_a_byte
rcall stop_bit
rjmp isr0_end


send_buttons:
lds r18, n64_buttons1
rcall send_a_byte
lds r18, n64_buttons2
rcall send_a_byte
lds r18, n64_x
rcall send_a_byte
lds r18, n64_y
rcall send_a_byte
rcall stop_bit
rjmp isr0_end


send_a_byte:
ldi r19, 8
next_bit:
sbi ddrd, 2 ;data=0
rcall delay7
nop
nop
nop
nop
nop
sbrs r18, 7 ;skip if data=1
rjmp kleiner_umweg ;if data=0, don't change data
cbi ddrd, 2 ;data=1
nop ;compensation nop
back_again:
rcall delay14
rcall delay14
nop
cbi ddrd, 2 ;data=1
lsl r18 ;leftshift databyte
dec r19 ;decrease count
breq byte_end ;if count=0, byte is complete
nop
rcall delay7
nop
rjmp next_bit
kleiner_umweg:
rjmp back_again
byte_end:
ret

stop_bit:
nop
nop
nop
sbi ddrd, 2
rcall delay14
cbi ddrd, 2
ret





toggle_led:
in r17, portc
ldi r16, 0b0000_0100
EOR r17, r16
out portc, r17
RET


;every new command must be started with a high to low transition of CSN (SS)
setup_nrf:
SET_CE
rcall wait10ms
RESET_CE
;setup rx
SET_SS
rcall delay7
RESET_SS
ldi r17, write_reg+rx_addr_p0
rcall do_spi
ldi r17, 0x34
rcall do_spi
ldi r17, 0x43
rcall do_spi
ldi r17, 0x10
rcall do_spi
ldi r17, 0x10
rcall do_spi
ldi r17, 0x01
rcall do_spi
SET_SS
rcall delay7
RESET_SS
ldi r17, write_reg+en_aa
rcall do_spi
ldi r17, 0x01
rcall do_spi
SET_SS
rcall delay7
RESET_SS
ldi r17, write_reg+en_rxaddr
rcall do_spi
ldi r17, 0x01
rcall do_spi
SET_SS
rcall delay7
RESET_SS
ldi r17, write_reg+rf_ch
rcall do_spi
LDS r17, CHANNEL ;LOAD CHANNEL
;ldi r17 , 50
rcall do_spi

SET_SS
rcall delay7
RESET_SS
ldi r17, write_reg+rx_pw_p0
rcall do_spi
;NEW-------------------------
ldi r17, 5	;OLD:3
rcall do_spi



SET_SS
rcall delay7
RESET_SS
ldi r17, write_reg+rf_setup
rcall do_spi
ldi r17, 0b0000_1110	;2MPS datarate
rcall do_spi

SET_SS
rcall delay7
RESET_SS
ldi r17, write_reg+config_nrf
rcall do_spi
ldi r17, 0x0F
rcall do_spi
SET_SS
rcall wait10ms
SET_CE
ret




do_spi:
	out spdr, r17 ;write out(x) in SPI data register
	spi_not_complete:
	sbis spsr, spif ;if SPI interrupt flag isn't set, then wait
	rjmp spi_not_complete
	in r17, spdr
ret


wait10ms:
	ldi r16, 10
	wait10times:
		ldi r31, high(4000)
		ldi r30, low(4000)
		msloop:
		sbiw r30, 1
		brne msloop
	dec r16
	brne wait10times
ret






translate_buttons:
;<------------- BUTTONS1--------------------->   <----------BUTTONS2-------------->
;UP DOWN LEFT RIGHT SELECT START    A     B   | | C    X    Y    Z   L1   L2   R1   R2
;7    6   5     4     3      2      1     0   | | 7    6    5    4   3    2    1    0
;********************************+******************************************************
;<------------- N641     -------------------->   <--------------N642------------------->
;A    B   Z   START   UP   DOWN   LEFT  RIGHT | | 0    0    L    R   C-U   C-D   C-L   C-R
;7    6   5     4     3      2      1     0   | | 7    6    5    4    3     2     1     0

;Buttons received by NRF loaded in r21 + r22
lds r21, buttons1
lds r22, buttons2
;***First N64 Buttons Byte:
ldi r16, 0xFF ;all buttons resetted
;A Button = A
sbrs r21, 1
cbr r16, 0b1000_0000
;B Button = X
sbrs r22, 6
cbr r16, 0b0100_0000
;Z Button = L2
sbrs r22, 2
cbr r16, 0b0010_0000
;Start Button = Start
sbrs r21, 2
cbr r16, 0b0001_0000
;Up = Up
sbrs r21, 7
cbr r16, 0b0000_1000
;Down = Down
sbrs r21, 6
cbr r16, 0b0000_0100
;Left = Left
sbrs r21, 5
cbr r16, 0b0000_0010
;Right = Right
sbrs r21, 4
cbr r16, 0b0000_0001
com r16 ;negate all the buttons because n64 wants it that way
sts n64_buttons1, r16 ;store the first button byte in n64_buttons1

;***Second N64 Buttons Byte:
ldi r16, 0xFF ;all buttons resetted
;L = L1
sbrs r22, 3
cbr r16, 0b0010_0000
;R = R1
sbrs r22, 1
cbr r16, 0b0001_0000
;C-Up = Z
sbrs r22, 4
cbr r16, 0b0000_1000
;C-Down = B
sbrs r21, 0
cbr r16, 0b0000_0100
;C-Left = Left
sbrs r22, 5
cbr r16, 0b0000_0010
;C-Right = C
sbrs r22, 7
cbr r16, 0b0000_0001
com r16 ;negate all the buttons because n64 wants it that way
sts n64_buttons2, r16 ;store the first button byte in n64_buttons2

;check, ob select gedrckt wurde:
	lds r16, buttons1
	sbrs r16, 3
	rjmp change_select_pressed
;select nicht gedrckt, dann gucken ob es das letzte mal gedrckt wurde
	lds r16, select_pressed
	ldi r17, 0x01
	sub r16, r17
	breq change_mode


;wenn MODE=1, dann DPAD->ANALOG STICK EMU:
	lds r16, mode_select
	ldi r17, 0x01
	sub r16, r17	;wenn mode=0, dann zu ende
	brne translate_buttons_end
;X-Axis
	ldi r16, 0x00
	ldi r17, 80
	sbrs r21, 5 
	sub r16, r17
	sbrs r21, 4
	add r16, r17
	sts n64_x, r16
;Y-Axis
	ldi r16, 0x00
	sbrs r21, 7 
	add r16, r17
	sbrs r21, 6
	sub r16, r17
	sts n64_y, r16
;Z bzw. L Knopf:
	lds r21, buttons2
	sbrc r21, 3	;wenn L(1) nicht gedrck wurde, dann ende
	rjmp translate_buttons_end
	;ansonsten:
	lds r16, n64_buttons1
	sbr r16, (1<<5)	;Z setzen
	sts n64_buttons1, r16
	lds r16, n64_buttons2
	cbr r16, (1<<5)	;L nicht setzen
	sts n64_buttons2, r16

	rjmp translate_buttons_end

change_select_pressed:
	ldi r16, 0x01
	sts select_pressed, r16
	rjmp translate_buttons_end

change_mode:
	lds r16, mode_select
	sbrs r16, 0		;setzen, wenn nicht gesetzt
	rjmp set_mode
	rjmp reset_mode
set_mode:
	ldi r16, 0x01
	sts mode_select, r16
	rjmp change_mode_end
reset_mode:
	ldi r16, 0x00
	sts mode_select, r16
	rjmp change_mode_end
change_mode_end:
	ldi r16, 0x00		;select_pressed zurcksetzen
	sts select_pressed, r16
	rjmp translate_buttons_end



translate_buttons_end:
	ret





















channel_select:
ldi r16, 109
sbis pind, 3
ldi r16, 119
sts channel, r16
ret

delay7:
ret
delay14:
rcall delay7
ret
delay16:
rcall delay7
nop
nop
ret
delay62:
rcall delay16
rcall delay16
rcall delay16
rcall delay7
ret







