.macro led_on
	cbi portd, 3
.endmacro

.macro led_off
	sbi portd, 3
.endmacro

.macro send0
	sbi ddrd, 2
	rcall delay10
	cbi ddrd, 2
	nop
	nop
.endmacro

.macro send1
	sbi ddrd, 2
	nop
	nop
	cbi ddrd, 2
	rcall delay10
.endmacro

.macro send80
	sbi ddrd, 2
	rcall delay10
	rcall delay10
	nop
	nop
	cbi ddrd, 2
	nop
	nop
	nop
	nop
	nop
	nop
.endmacro

.macro send81
	sbi ddrd, 2
	nop
	nop
	nop
	nop
	nop
	nop
	cbi ddrd, 2
	rcall delay10
	rcall delay10
	nop
	nop
.endmacro


.nolist
.include "tn2313def.inc"
.include "nrf24l01.inc"
.list

.equ sck = 7
.equ mosi = 6
.equ miso = 5
.equ ss = 4
.equ ce = 3
.equ irq = 2

.macro set_ss
sbi portb, ss
.endmacro

.macro reset_ss
cbi portb, ss
.endmacro

.macro set_ce
sbi portb, ce
.endmacro

.macro reset_ce
cbi portb, ce
.endmacro



.cseg
.org 0000
rjmp init ;reset 
reti ;int0
reti ;int1
reti ;timer1 capt
reti ;timer1 compa
rjmp timer1_ovf;reti ;timer1 ovf 
reti ;timer0 ovf
reti; usart0, rx
reti ;usart0, udre
reti ;usart0, tx
reti ;analog comp
reti ;pcint
reti ;timer1 compb
reti ;timer0 compa
reti ;timer0 compb
reti ;usi start
reti ;usi overflow
reti ;eeprom ready
reti ;wdt overflow


init:
;DISABLE ANALOG COMPARATROR
ldi r16, 0b1000_0000
out acsr, r16

;INIT STACK
ldi r16, LOW(RAMEND) ; Unteres Byte
out SPL, r16 ; an Stapelzeiger


;Buttons Variables
.equ id = SRAM_START
ldi r16, 0x03	;ID=3 => N64
sts id, r16
.equ buttons1 = SRAM_START+1
.equ buttons2 = SRAM_START+2

.equ n64_1 = SRAM_START+5
.equ n64_2 = SRAM_START+6
.equ n64_3 = SRAM_START+7
.equ n64_4 = SRAM_START+8
.equ channel = SRAM_START+9

.equ timer_value_8mhz = 0xBEE8	;=60Hz, wenn f=clk/8

.def poll_again= r23
clr poll_again






;CONFIG NRF PINS
;SCK output ;MISO input;MOSI output;SS output;CE output;IRQ input
LDI R16,(1<<sck)|(1<<mosi)|(1<<ss)|(1<<ce)
out DDRB, r16
set_ss


rcall set_channel

;CONFIG NRF
rcall setup_nrf
;FLUSH TX FIFO
RESET_SS
ldi r17, flush_tx
rcall do_spi
SET_SS
;CLEAR IRQ
RESET_SS
ldi r17, write_reg+status
rcall do_spi
ldi r17, 0b0011_0000
rcall do_spi
SET_SS


;-----------------------------------
;SET UP TIMER1----------------------
ldi r16, high(timer_value_8mhz)
out TCNT1H, r16
ldi r16, low(timer_value_8mhz)
out TCNT1L, r16
 
ldi r16, 0b0000_0010	;f=CLK/8
out TCCR1B, r16
 
ldi r16, 0b10000000
out TIMSK, r16
;-----------------------------------
;END TIMER1 SETUP-------------------

rcall wait10ms
rcall wait10ms
rcall wait10ms
sei



 
main:

	;wenn poll_gain = 0, dann warten und zurck auf main springen
	sbrs poll_again, 0
	rjmp main	
	clr poll_again ;poll_gain zurcksetzen. wird von timer1_ovf gesetzt, nach 16,67ms

	
	cli	;interrupts verbieten, solange pad abgefragt wird, sonst wird das timing verhunzt
	rcall read_n64pad	;n64 pad abfragen
	sei	;interrupts wieder erlauben	
	
	;TRANSLATE BUTTONS			
	rcall translate_buttons

	;Write Payload
	RESET_SS
	ldi r17, wr_tx_pload
	rcall do_spi
	lds r17, id
	rcall do_spi
	lds r17, buttons1 ;buttons1
	rcall do_spi
	lds r17, buttons2 ;buttons2
	rcall do_spi
	;--------------------
	;NEW: analog bytes abfragen
	lds r17, n64_3
	rcall do_spi
	lds r17, n64_4
	rcall do_spi
	;--------------------
	SET_SS
	;transmit now
	SET_CE
	rcall delay84
	RESET_CE

	;***wait until IRQ=0
	wait_for_IRQ:
	sbic pinb, irq
	rjmp wait_for_irq
	;***clear IRQ

	;FLUSH TX FIFO
	RESET_SS
	ldi r17, flush_tx
	rcall do_spi
	SET_SS
	;CLEAR IRQ
	RESET_SS
	ldi r17, write_reg+status
	rcall do_spi
	ldi r17, 0b0011_0000
	rcall do_spi
	SET_SS

	;go to sleep:
	ldi r17, 0b0010_0000
	out mcucr, r17	;enable idle
	sleep
	

	rjmp main


;------------SETUP NRF--------------------
;-----------------------------------------
setup_nrf:
		SET_CE
		rcall wait10ms
		RESET_CE
		RESET_SS
		ldi r17, write_reg+tx_addr
		rcall do_spi
		ldi r17, 0x34
		rcall do_spi
		ldi r17, 0x43
		rcall do_spi
		ldi r17, 0x10
		rcall do_spi
		ldi r17, 0x10
		rcall do_spi
		ldi r17, 0x01
		rcall do_spi

		SET_SS
		RESET_SS
		ldi r17, write_reg+rx_addr_p0
		rcall do_spi
		ldi r17, 0x34
		rcall do_spi
		ldi r17, 0x43
		rcall do_spi
		ldi r17, 0x10
		rcall do_spi
		ldi r17, 0x10
		rcall do_spi
		ldi r17, 0x01
		rcall do_spi

		SET_SS
		RESET_SS
		ldi r17, write_reg+en_aa ;enable auto 
		rcall do_spi
		ldi r17, 0x01
		rcall do_spi

		SET_SS
		RESET_SS
		ldi r17, write_reg+en_rxaddr
		rcall do_spi
		ldi r17, 0x01
		rcall do_spi

		SET_SS
		RESET_SS
		ldi r17, write_reg+rf_ch
		rcall do_spi
		lds r17, channel 
		;ldi r17, 50 ;Channel
		rcall do_spi


		SET_SS
		RESET_SS
		ldi r17, write_reg+rf_setup
		rcall do_spi
		ldi r17, 0b0000_1110	;2MPS datarate
		rcall do_spi

		SET_SS
		RESET_SS
		ldi r17, write_reg+setup_retr
		rcall do_spi
		ldi r17, 0b0001_1010 
		rcall do_spi

		SET_SS
		RESET_SS
		ldi r17, write_reg+config_nrf
		rcall do_spi
		ldi r17, 0x0E
		rcall do_spi
		SET_SS
		ret
;------------------ENDE SETUP NRF----------------
;-------------------------------------------------



;-----------------------------------
;-----------------------------------
;READ N64 PAD:
read_n64pad:
					clr r16	;register lschen, da sonst buttons "so" gedrckt werden.

					send80
					send80
					send80
					send80
					send80
					send80
					send80
					send81
					send81
wait:
					sbic pind, 2
					rjmp wait

					rcall delay7

					;FIRST BYTE
					sbic pind,2
					sbr r16, (1<<0)
					rcall delay30

					sbic pind,2
					sbr r16, (1<<1)
					rcall delay30

					sbic pind,2
					sbr r16, (1<<2)
					rcall delay30

					sbic pind,2
					sbr r16, (1<<3)
					rcall delay30

					sbic pind,2
					sbr r16, (1<<4)
					rcall delay30

					sbic pind,2
					sbr r16, (1<<5)
					rcall delay30

					sbic pind,2
					sbr r16, (1<<6)
					rcall delay30

					sbic pind,2
					sbr r16, (1<<7)
					sts n64_1, r16	;byte abspeichern
					clr r16			;r16 lschen
					rcall delay27	;nur 27clks warten

					;SECOND BYTE
					sbic pind,2
					sbr r16, (1<<0)
					rcall delay30

					sbic pind,2
					sbr r16, (1<<1)
					rcall delay30

					sbic pind,2
					sbr r16, (1<<2)
					rcall delay30

					sbic pind,2
					sbr r16, (1<<3)
					rcall delay30

					sbic pind,2
					sbr r16, (1<<4)
					rcall delay30

					sbic pind,2
					sbr r16, (1<<5)
					rcall delay30

					sbic pind,2
					sbr r16, (1<<6)
					rcall delay30

					sbic pind,2
					sbr r16, (1<<7)
					sts n64_2, r16	;2. byte abspeichern
					clr r16			;r16 lschen
					rcall delay27	;nur 27 clks warten

					;THIRD BYTE
					sbic pind,2
					sbr r16, (1<<7)
					rcall delay30

					sbic pind,2
					sbr r16, (1<<6)
					rcall delay30

					sbic pind,2
					sbr r16, (1<<5)
					rcall delay30

					sbic pind,2
					sbr r16, (1<<4)
					rcall delay30

					sbic pind,2
					sbr r16, (1<<3)
					rcall delay30

					sbic pind,2
					sbr r16, (1<<2)
					rcall delay30

					sbic pind,2
					sbr r16, (1<<1)
					rcall delay30

					sbic pind,2
					sbr r16, (1<<0)
					sts n64_3, r16	;3. byte abspeichern
					clr r16			;r16 lschen
					rcall delay27	;nur 27 clks warten

					;FOURTH BYTE
					sbic pind,2
					sbr r16, (1<<7)
					rcall delay30

					sbic pind,2
					sbr r16, (1<<6)
					rcall delay30

					sbic pind,2
					sbr r16, (1<<5)
					rcall delay30

					sbic pind,2
					sbr r16, (1<<4)
					rcall delay30

					sbic pind,2
					sbr r16, (1<<3)
					rcall delay30

					sbic pind,2
					sbr r16, (1<<2)
					rcall delay30

					sbic pind,2
					sbr r16, (1<<1)
					rcall delay30

					sbic pind,2
					sbr r16, (1<<0)
					sts n64_4, r16	;4. byte speichern
					

					ret
;----------------------------------------------------
;---------ENDE READ_N64_PAD--------------------------



;--------------------------------------------------------
;--------------TRANSLATE BUTTONS:------------------------
translate_buttons:
;<------------- BUTTONS1--------------------->   <----------BUTTONS2-------------->
;UP DOWN LEFT RIGHT SELECT START    A     B   | | C    X    Y    Z   L1   L2   R1   R2
;7    6   5     4     3      2      1     0   | | 7    6    5    4   3    2    1    0


;<--------------N64_1------------------------>   <------------N64_2--------------->                                           
;7    6   5     4     3      2      1     0   | | 7    6    5    4   3    2    1    0
;RT LEFT DOWN  UP   START    Z      B     A   | |C-R  C-L  C-D  C-U  R    L    NA   NA

		
		;controller data laden:
		lds r16, n64_1
		lds r17, n64_2

		;buttons1 = r20, buttons2=r21
		;buttons1+2 zurcksetzen -> 0xff
		ser r20 ;buttons1
		ser r21 ;buttons2


		;N64_1 zuweisen:
		sbrc r16, 0 ;N64 A = A
		cbr r20, (1<<1)
		sbrc r16, 1 ;N64 B = X
		cbr r21, (1<<6)
		sbrc r16, 2 ;N64 Z = L2
		cbr r21, (1<<2)
		sbrc r16, 3 ;N64 START = START
		cbr r20, (1<<2)
		sbrc r16, 4 ;N64 UP = UP
		cbr r20, (1<<7)
		sbrc r16, 5 ;N64 DOWN = DOWN
		cbr r20, (1<<6)
		sbrc r16, 6 ;N64 LEFT = LEFT
		cbr r20, (1<<5)
		sbrc r16, 7 ;N64 RIGHT = RIGHT
		cbr r20, (1<<4)

		;N64_2 zuweisen:
		sbrc r17, 2 ;N64 L = L1
		cbr r21, (1<<3)
		sbrc r17, 3 ;N64 R = R1
		cbr r21, (1<<1)
		sbrc r17, 4 ;N64 C-UP = Z
		cbr r21, (1<<4)
		sbrc r17, 5 ;N64 C-DOWN = B
		cbr r20, (1<<0)
		sbrc r17, 6 ;N64 C-LEFT = Y
		cbr r21, (1<<5)
		sbrc r17, 7 ;N64 C-RIGHT = C
		cbr r21, (1<<7)
		
		;buttons abspeichern:
		sts buttons1, r20
		sts buttons2, r21

			

		ret
;---------------------------------------------------------
;----------ENDE TRANSLATE BUTTONS:------------------------



;------DO SPI ROUTINE--------------------------------------------------------
;------INPUT & OUTPUT in R17!!-----------------------------------------------
do_spi:
					out USIDR, r17
					ldi r16, (1<<USIWM0)|(0<<USICS0)|(1<<USITC) 
					ldi r17, (1<<USIWM0)|(0<<USICS0)|(1<<USITC)|(1<<USICLK) 

					out USICR, r16 ;MSB
					out USICR, r17
					out USICR, r16 
					out USICR, r17
					out USICR, r16 
					out USICR, r17
					out USICR, r16 
					out USICR, r17
					out USICR, r16 
					out USICR, r17
					out USICR, r16 
					out USICR, r17
					out USICR, r16 
					out USICR, r17
					out USICR, r16 ;LSB 
					out USICR, r17

					in r17, USIDR
					ret
;----------------------------------------------------------------------------
;-------------ENDE SPI ROUTINE-----------------------------------------------



;----------------------------------------
;----------------WARTE ROUTINEN----------
delay7:
	ret
delay10:
	nop
	nop
	nop
	ret
delay14:
	rcall delay7
	ret
delay27:
	rcall delay10
	rcall delay10
	ret
delay30:
	rcall delay10
	rcall delay10
	nop
	nop
	nop
	ret
delay84:
	rcall delay14
	rcall delay14
	rcall delay14
	rcall delay14
	rcall delay14
	rcall delay7
	ret
wait10ms:
	ldi r16, 10
	wait10times:
		ldi r31, high(4000)
		ldi r30, low(4000)
		msloop:
		sbiw r30, 1
		brne msloop
	dec r16
	brne wait10times
ret
;---------------ENDE WARTE ROUTINEN----------
;--------------------------------------------



;---------SET CHANNEL:----------------------
;-------------------------------------------
set_channel:
rcall wait10ms

;rjmp set_channel_2
	rcall read_n64pad	;Pad lesen
	rcall translate_buttons
	lds r17, n64_2		;2. N64 byte laden
	com r17				;negieren
	ldi r18, 0xFF		;buttons2 nicht gedrckt: weiter.	buttons2 gedrckt: abbruch, channel von eeprom lesen
	sub r17, r18
	brne read_channel

	lds r17, n64_1		;1. n64 byte laden
	com r17				;negieren
	ldi r18, 0b1111_1110	;gucken ob (ausschlielich) A gedrckt wurde
	sub r17, r18
	breq set_channel_1	;wenn nur A, dann set channel 1
	lds r17, n64_1		;1. byte
	com r17				;negieren
	ldi r18, 0b1111_1101	;gucken ob nur B gedrckt wurde
	sub r18, r17
	breq set_channel_2

read_channel:
	cli						;interrupts abschalten
	sbic EECR, EEpE			;schauen ob EEPROM busy ist
	rjmp read_channel		;wenn nicht, warten
	ldi r17, 0x08			;eeprom addresse laden
	out EEAR, r17			;addresse ins eeprom adress register laden
	sbi EECR, EERE			;read enable bit setzen
	in r17, EEDR			;channel aus eeprom laden
	sts channel, r17
	rjmp set_channel_end	;set_channel beenden
	sei						;interrupts wieder erlauben

set_channel_1:				;channel = 109
	ldi r17, 109
	sts channel, r17
	rjmp store_channel

set_channel_2:				;channel = 119
	ldi r17, 119
	sts channel, r17
	rjmp store_channel

store_channel:
	cli						;interrupts ausschalten
	sbic EECR, EEPE			;wenn write enable = 0, dann ist eeprom bereit beschrieben zu werden
	rjmp store_channel		;ansonsten loopen bis dahein
	ldi r17, 0x08			;lade eeprom adresse in r17
	out EEAR, r17			; schreibe adresse in address register
	lds r17, channel		;channel in r17 laden
	out EEDR, r17			;data register mit channel laden
	sbi EECR, EEMPE			;master write enable bit setzen
	sbi EECR, EEPE			;write enable bit setzen
	sei						;interupts wieder anschalten

set_channel_end:
	ret
;------------SET CHANNEL END---------------------------------------------------------
;--------------------------------------------------------------------------------------



;----------------RESET N64 CONTROLLER------------------
;------------------------------------------------------
reset_n64pad:
	send81
	send81
	send81
	send81
	send81
	send81
	send81
	send81
	send81
	ret
;-------------RESET CONTROLLER END------------------
;---------------------------------------------------



;------TIMER1 OVERFLOW-------------------------------
;----------------------------------------------------
timer1_ovf:
		push r16	;save r16
		in r16, sreg
		push r16	;save sreg	

		ldi r16, high(timer_value_8mhz) ;relaod timer1
		out TCNT1H, r16
		ldi r16, low(timer_value_8mhz)
		out TCNT1L, r16

		ser poll_again	;16,67ms verstrichen, pad wieder abfragen

		pop r16
		out sreg, r16 ;restore sreg
		pop r16			;restore r16
			
		
		reti	;return from interrupt
;---------------------TIMER1 OVERFLOW END------------
;-----------------------------------------------------








