/*
 * NES_TX1.asm
 *
 *  Created: 10.04.2012 15:17:18
 *   Author: Jakob Schafer
 ID:	0x01	SNES
		0x02	SATURN
		0x03	N64
		0x04	NES
 */ 

 ;TX2
;NEW: AUTO TURNOFF NACH EINIGEN SEKUNDEN

.nolist
.include "tn2313def.inc"
.include "nrf24l01.inc"
.list

.equ sck = 7
.equ mosi = 6
.equ miso = 5
.equ ss = 4
.equ ce = 3
.equ irq = 2

;---------NRF MACROS--------
;---------------------------
.macro set_ss
	sbi portb, ss
.endmacro

.macro reset_ss
	cbi portb, ss
.endmacro

.macro set_ce
	sbi portb, ce
.endmacro

.macro reset_ce
	cbi portb, ce
.endmacro
;--------SNES MACROS--------
;---------------------------
.macro set_latch
	sbi portd, nes_latch
.endmacro

.macro reset_latch
	cbi portd, nes_latch
.endmacro

.macro set_clock
	sbi portd, nes_clock		;set_clock
.endmacro

.macro reset_clock
	cbi portd, nes_clock		;reset_clock
.endmacro

.macro set_vcc
	sbi portd, nes_vcc
.endmacro

.macro reset_vcc
	cbi portd, nes_vcc
.endmacro

.macro delay6
	nop
	nop
	nop
	nop
	nop
	nop
.endmacro

.macro delay4
	nop
	nop
	nop
	nop
.endmacro

.macro delay2
	nop
	nop
.endmacro





.cseg
.org 0000
rjmp init ;reset 
reti ;int0
reti ;int1
reti ;timer1 capt
reti ;timer1 compa
rjmp timer1_ovf;reti ;timer1 ovf 
reti ;timer0 ovf
reti; usart0, rx
reti ;usart0, udre
reti ;usart0, tx
reti ;analog comp
reti ;pcint
reti ;timer1 compb
reti ;timer0 compa
reti ;timer0 compb
reti ;usi start
reti ;usi overflow
reti ;eeprom ready
reti ;wdt overflow


init:
;DISABLE ANALOG COMPARATROR
ldi r16, 0b1000_0000
out acsr, r16

;INIT STACK
ldi r16, LOW(RAMEND) ; Unteres Byte
out SPL, r16 ; an Stapelzeiger


;Buttons Variables
.equ id = SRAM_START
ldi r16, 0x04	;ID=4 => NES
sts id, r16
.equ buttons1 = SRAM_START+1
.equ buttons2 = SRAM_START+2
.equ old_buttons1 = SRAM_START+3
.equ old_buttons2 = SRAM_START+4
.equ counter1 = SRAM_START + 5
.equ counter2 = SRAM_START + 6
.equ dont_send = sram_start+7
.equ channel = SRAM_START+9

.equ timer_value_1mhz = 0xBEF9	;=60Hz, wenn f=clk/8

.def poll_again= r23
clr poll_again



;CONFIG NES PINS
.equ nes_vcc = 2
.equ nes_clock = 3
.equ nes_latch = 4
.equ nes_data = 5

;latch, clock, vcc = output
ldi r16, (1<<nes_latch)|(1<<nes_clock)|(1<<nes_vcc)
out ddrd, r16
reset_latch
reset_clock
reset_vcc



;CONFIG NRF PINS
;SCK output ;MISO input;MOSI output;SS output;CE output;IRQ input
LDI R16,(1<<sck)|(1<<mosi)|(1<<ss)|(1<<ce)
out DDRB, r16
set_ss


rcall set_channel

;CONFIG NRF
rcall setup_nrf
;FLUSH TX FIFO
RESET_SS
ldi r17, flush_tx
rcall do_spi
SET_SS
;CLEAR IRQ
RESET_SS
ldi r17, write_reg+status
rcall do_spi
ldi r17, 0b0011_0000
rcall do_spi
SET_SS


;-----------------------------------
;SET UP TIMER1----------------------
ldi r16, high(timer_value_1mhz)
out TCNT1H, r16
ldi r16, low(timer_value_1mhz)
out TCNT1L, r16
 
ldi r16, 0b0000_0001	;f=CLK/1
out TCCR1B, r16
 
ldi r16, 0b10000000
out TIMSK, r16	
;-----------------------------------
;END TIMER1 SETUP-------------------


sei						;enable interrupts



 
main:

	;wenn poll_gain = 0, dann warten und zurck auf main springen
	sbrs poll_again, 0
	rjmp main	
	clr poll_again ;poll_gain zurcksetzen. wird von timer1_ovf gesetzt, nach 16,67ms

	
	rcall read_nespad	;NES pad abfragen
	
	rcall new_buttons	;checken ob alte daten, dann nicht senden
	lds r16, dont_send
	ldi r17, 0x01
	sub r17, r16
	breq GO_TO_SLEEP	;wenn dont_send=1 => nicht senden, zurck zu main

SENDEN:
	;Write Payload
	RESET_SS
	ldi r17, wr_tx_pload
	rcall do_spi
	lds r17, ID			;ID
	rcall do_spi
	lds r17, buttons1	;buttons1
	rcall do_spi
	lds r17, buttons2	;buttons2
	rcall do_spi
	;--------------------
	;NEW: analog bytes abfragen
	ldi r17, 0xff		;x axis
	rcall do_spi
	ldi r17, 0xff		;y axis
	rcall do_spi
	;--------------------
	SET_SS
	;transmit now
	SET_CE
	rcall delay84
	RESET_CE

	;***wait until IRQ=0
	wait_for_IRQ:
	sbic pinb, irq
	rjmp wait_for_irq
	;***clear IRQ

	;FLUSH TX FIFO
	RESET_SS
	ldi r17, flush_tx
	rcall do_spi
	SET_SS
	;CLEAR IRQ
	RESET_SS
	ldi r17, write_reg+status
	rcall do_spi
	ldi r17, 0b0011_0000
	rcall do_spi
	SET_SS

	;go to sleep:
GO_TO_SLEEP:
	ldi r17, 0b0010_0000
	out mcucr, r17	;enable idle
	sleep
	

	rjmp main


;------------SETUP NRF--------------------
;-----------------------------------------
setup_nrf:
		SET_CE
		rcall wait10ms
		RESET_CE
		RESET_SS
		ldi r17, write_reg+tx_addr
		rcall do_spi
		ldi r17, 0x34
		rcall do_spi
		ldi r17, 0x43
		rcall do_spi
		ldi r17, 0x10
		rcall do_spi
		ldi r17, 0x10
		rcall do_spi
		ldi r17, 0x01
		rcall do_spi

		SET_SS
		RESET_SS
		ldi r17, write_reg+rx_addr_p0
		rcall do_spi
		ldi r17, 0x34
		rcall do_spi
		ldi r17, 0x43
		rcall do_spi
		ldi r17, 0x10
		rcall do_spi
		ldi r17, 0x10
		rcall do_spi
		ldi r17, 0x01
		rcall do_spi

		SET_SS
		RESET_SS
		ldi r17, write_reg+en_aa ;enable auto 
		rcall do_spi
		ldi r17, 0x01
		rcall do_spi

		SET_SS
		RESET_SS
		ldi r17, write_reg+en_rxaddr
		rcall do_spi
		ldi r17, 0x01
		rcall do_spi

		SET_SS
		RESET_SS
		ldi r17, write_reg+rf_ch
		rcall do_spi
		lds r17, channel 
		;ldi r17, 101 ;Channel
		rcall do_spi


		SET_SS
		RESET_SS
		ldi r17, write_reg+rf_setup
		rcall do_spi
		ldi r17, 0b0000_1110	;2MPS datarate
		rcall do_spi

		SET_SS
		RESET_SS
		ldi r17, write_reg+setup_retr
		rcall do_spi
		ldi r17, 0b0001_1010
		rcall do_spi

		SET_SS
		RESET_SS
		ldi r17, write_reg+config_nrf
		rcall do_spi
		ldi r17, 0x0E
		rcall do_spi
		SET_SS
		ret
;------------------ENDE SETUP NRF----------------
;-------------------------------------------------



;-----------------------------------
;-----------------------------------
;READ NES PAD:
;<------------- BUTTONS1--------------------->   <----------BUTTONS2-------------->
;UP DOWN LEFT RIGHT SELECT START    A     B   | | C    X    Y    Z   L1   L2   R1   R2
;7    6   5     4     3      2      1     0   | | 7    6    5    4   3    2    1    0

;NES ORDER:
;A			B		SELECT		START		UP			DOWN		LEFT		RIGHT 

read_nespad:
		set_vcc					;power up the nes pad
		set_clock				;clock auf high
		ser r16					;clr buttons1
		ser r17					;clr buttons2

		set_latch				;12us positive latch pulse
		rcall delay10
		nop
		nop
		reset_latch

		delay6					;wait another 6 us

		sbis pind, nes_data		;NES A = B
		cbr r16, (1<<0)
		reset_clock
		delay6

		set_clock
		delay2
		sbis pind, nes_data		;NES B = A
		cbr r16, (1<<1)
		reset_clock
		delay4

		set_clock
		delay2
		sbis pind, nes_data		;NES SELECT = SELECT
		cbr r16, (1<<3)
		reset_clock
		delay4

		set_clock
		delay2
		sbis pind, nes_data		;NES START = START
		cbr r16, (1<<2)
		reset_clock
		delay4

		set_clock
		delay2
		sbis pind, nes_data		;NES UP = UP
		cbr r16, (1<<7)
		reset_clock
		delay4

		set_clock
		delay2
		sbis pind, nes_data		;NES DOWN = DOWN
		cbr r16, (1<<6)
		reset_clock
		delay4

		set_clock
		delay2
		sbis pind, nes_data		;NES LEFT = LEFT
		cbr r16, (1<<5)
		reset_clock
		delay4

		set_clock
		delay2
		sbis pind, nes_data		;NES RIGHT = RIGHT
		cbr r16, (1<<4)
		reset_clock
		delay4

	
		;TURN OF THE NES PAD
		reset_clock
		reset_latch
		reset_vcc		

		sts buttons1, r16		;store buttons1
		ldi r16, 0xFF			;Immer gilt: buttons2 = 0xFF
		sts buttons2, r16
		ret 
;----------------------------------------------------
;---------ENDE READ_NESPAD--------------------------



;------DO SPI ROUTINE--------------------------------------------------------
;------INPUT & OUTPUT in R17!!-----------------------------------------------
do_spi:
					out USIDR, r17
					ldi r16, (1<<USIWM0)|(0<<USICS0)|(1<<USITC) 
					ldi r17, (1<<USIWM0)|(0<<USICS0)|(1<<USITC)|(1<<USICLK) 

					out USICR, r16 ;MSB
					out USICR, r17
					out USICR, r16 
					out USICR, r17
					out USICR, r16 
					out USICR, r17
					out USICR, r16 
					out USICR, r17
					out USICR, r16 
					out USICR, r17
					out USICR, r16 
					out USICR, r17
					out USICR, r16 
					out USICR, r17
					out USICR, r16 ;LSB 
					out USICR, r17

					in r17, USIDR
					ret
;----------------------------------------------------------------------------
;-------------ENDE SPI ROUTINE-----------------------------------------------



;----------------------------------------
;----------------WARTE ROUTINEN----------
delay7:
	ret
delay10:
	nop
	nop
	nop
	ret
delay14:
	rcall delay7
	ret
delay27:
	rcall delay10
	rcall delay10
	ret
delay30:
	rcall delay10
	rcall delay10
	nop
	nop
	nop
	ret
delay84:
	rcall delay14
	rcall delay14
	rcall delay14
	rcall delay14
	rcall delay14
	rcall delay7
	ret
wait10ms:
	ldi r16, 10
	wait10times:
		ldi r31, high(4000)
		ldi r30, low(4000)
		msloop:
		sbiw r30, 1
		brne msloop
	dec r16
	brne wait10times
ret
;---------------ENDE WARTE ROUTINEN----------
;--------------------------------------------



;---------SET CHANNEL:----------------------
;-------------------------------------------
set_channel:

	rcall read_nespad		;Pad lesen

	lds r17, buttons2		;buttons2 laden
	ldi r18, 0xFF			;wenn buttons2 nicht ungedrckt, dann channel lesen
	sub r17, r18
	brne read_channel


	lds r17, buttons1		;buttons1 laden
	ldi r18, 0b1111_1110	;wenn nur NES-A gedrckt, dann channel 1
	sub r18, r17
	breq set_channel_1

	lds r17, buttons1		;buttons1 laden
	ldi r18, 0b1111_1101	;wenn nur NES-B gedrckt, dann channel 2
	sub r18, r17
	breq set_channel_2
	
read_channel:
	cli						;interrupts abschalten
	sbic EECR, EEpE			;schauen ob EEPROM busy ist
	rjmp read_channel		;wenn nicht, warten
	ldi r17, 0x08			;eeprom addresse laden
	out EEAR, r17			;addresse ins eeprom adress register laden
	sbi EECR, EERE			;read enable bit setzen
	in r17, EEDR			;channel aus eeprom laden
	sts channel, r17
	rjmp set_channel_end	;set_channel beenden
	sei						;interrupts wieder erlauben

set_channel_1:				;channel = 109
	ldi r17, 109
	sts channel, r17
	rjmp store_channel

set_channel_2:				;channel = 119
	ldi r17, 119
	sts channel, r17
	rjmp store_channel

store_channel:
	cli						;interrupts ausschalten
	sbic EECR, EEPE			;wenn write enable = 0, dann ist eeprom bereit beschrieben zu werden
	rjmp store_channel		;ansonsten loopen bis dahein
	ldi r17, 0x08			;lade eeprom adresse in r17
	out EEAR, r17			; schreibe adresse in address register
	lds r17, channel		;channel in r17 laden
	out EEDR, r17			;data register mit channel laden
	sbi EECR, EEMPE			;master write enable bit setzen
	sbi EECR, EEPE			;write enable bit setzen
	sei						;interupts wieder anschalten

set_channel_end:
	ret
;------------SET CHANNEL END---------------------------------------------------------
;--------------------------------------------------------------------------------------



;------TIMER1 OVERFLOW-------------------------------
;----------------------------------------------------
timer1_ovf:
		push r16	;save r16
		in r16, sreg
		push r16	;save sreg	

		ldi r16, high(timer_value_1mhz) ;relaod timer1
		out TCNT1H, r16
		ldi r16, low(timer_value_1mhz)
		out TCNT1L, r16

		ser poll_again	;16,67ms verstrichen, pad wieder abfragen

		pop r16
		out sreg, r16 ;restore sreg
		pop r16			;restore r16
			
		
		reti	;return from interrupt
;---------------------TIMER1 OVERFLOW END------------
;-----------------------------------------------------



;------------------------------------------------
;----------CHECK NEW BUTTONS-----------------
NEW_BUTTONS:
	lds r16, buttons1
	lds r17, old_buttons1
	sub r16, r17
	brne RESET_COUNTER	;wenn ungleich => andere buttons, counter zurcksetzen
	lds r16, buttons2
	lds r17, old_buttons2
	sub r16, r17
	brne RESET_COUNTER	;wenn ungleich => andere buttons, counter zurcksetzen

	lds r16, dont_send	;wenn dont_send =1, dann gleich zum ende, dann brauchst du gar net erst gucken
	ldi r17, 0x01
	sub r17, r16
	breq NEW_BUTTONS_END 

INCREASE_FRAME_COUNTER:	;increase frame counter
	lds r16, counter1
	inc r16
	sts counter1, r16
	ldi r17, 60 ;60 frames=1sec
	sub r17, r16
	breq INCREASE_SECOND_COUNTER
	rjmp NEW_BUTTONS_END

INCREASE_SECOND_COUNTER:	;increase seconds counter
	clr r16
	sts counter1, r16	;framecounter lschen
	lds r16, counter2
	inc r16
	sts counter2, r16
	ldi r17, 5	;5 sekunden
	sub r17, r16
	brne NEW_BUTTONS_END	;wenn nicht 5 sekunden, dann ende
	ldi r16, 0x01
	sts dont_send, r16		;wenn 5. sekunde, dann nichtsenden=1
	rjmp NEW_BUTTONS_END


RESET_COUNTER:
	clr r16
	sts counter1, r16
	sts counter2, r16
	sts dont_send, r16
	rjmp NEW_BUTTONS_END

NEW_BUTTONS_END:
	lds r16, buttons1
	sts old_buttons1, r16
	lds r16, buttons2
	sts old_buttons2, r16
	ret











