.nolist
.include "m8def.inc"
.include "nrf24l01.inc"
.list

.cseg
.org 0000
rjmp init ;reset 
rjmp isr0 ;int0
reti ;int1
reti ;timer2 comp
reti ;timer2 overflow
reti ;timer1 capture
reti ;timer1 compare A
reti; timer1 compare N
reti ;timer1 overflow
reti ;timer0 overflow
reti ;SPI serial transfer complete
reti ;USART rx complete
reti ;USART DR empty
reti ;USART tx complete
reti ;ADC complete
reti ;eeprom ready
reti ;analog comparator
reti ;two wire interface
reti ;store program memory ready


init:
;INIT STACK
	ldi r16, HIGH(RAMEND) ; Oberes Byte
	out SPH, r16 ; an Stapelzeiger
	ldi r16, LOW(RAMEND) ; Unteres Byte
	out SPL, r16 ; an Stapelzeiger

;INIT SATURN REGS
	.def satll=r18 ;R_X_Y_Z
	.def sathl=r19 ;START_A_C_B
	.def satlh=r20 ;R_L_D_U
	.def sathh=r21 ;L_1_0_0
	ldi satll, 0b0001_1111
	ldi sathl, 0b0001_1111
	ldi satlh, 0b0001_1111
	ldi sathh, 0b0001_1100

;CONFIGURE SAT PINS
	.equ channel_switch = 0 ;PORTD.0
	.equ s0 = 2
	.equ s1 = 3
	.equ d0 = 0
	.equ d1 = 1
	.equ d2 = 2
	.equ d3 = 3
	.equ sat_vcc = 4
;d0:d3 + sat_vcc = output
	ldi r16, (1<<d0)|(1<<d1)|(1<<d2)|(1<<d3)|(1<<sat_vcc)
	out ddrc, r16
;pull ups for s0 + s1 + channel switch
	ldi r16, (1<<s0) | (1<<s1) | (1<<channel_switch)
	out portd, r16

;VARIABLES FOR RECEIVED BUTTONS
	.equ ID = SRAM_START+1
	.equ BUTTONS1 = SRAM_START+2
	.equ BUTTONS2 = SRAM_START+3
	.equ CHANNEL = SRAM_START+4

	rcall channel_select


;CONFIG SPI PINS
;SCK output ;MISO input;MOSI output;SS output;CE output;IRQ input
	.equ ce = 0
	.equ irq = 1
	.equ ss = 2
	.equ mosi = 3
	.equ miso = 4
	.equ sck = 5
	LDI R16,(1<<ce)|(1<<ss)|(1<<mosi)|(1<<sck)
	out DDRB, r16
;Pullups for IRQ + MISO
	ldi r16, (1<<miso)|(1<<irq)
	out PORTB, r16

;CONFIG SPI
; Enable SPI, Master, clockrate f/4, MSB, 
ldi r17,(1<<SPE)|(1<<MSTR)
out SPCR,r17


;CONFIG INT0
ldi r16, 0x01
out MCUCR, r16
ldi r16, 0b0100_0000
out GICR, r16
sei

;CONFIG NRF
rcall setup_nrf



main:
;***CHECK IF CHANNEL HAS BEEN CHANGED
check_channel:
	lds r16, channel
	sbis pind,channel_switch
	rjmp channel_switch_high

channel_switch_low:
	ldi r17, 109
	sub r17, r16
	BRNE wrong_channel ;FALSCHER KANAL
	RJMP check_channel_end ;RICHTIGER KANAL

channel_switch_high:
	ldi r17, 119
	sub r17, r16
	BRNE wrong_channel
	RJMP check_channel_end

wrong_channel:
	rcall channel_select
	rcall setup_nrf

check_channel_end:


;IF IRQ=1 THEN GOTO MAIN
	in r16, pinb
	sbrc r16, 1
	rjmp main
;IF IRQ=0 THEN:
	RESET_CE
read_fifo:
	RESET_SS
	ldi r17, rd_rx_pload
	rcall do_spi
	rcall do_spi
	sts ID, r17
	rcall do_spi
	sts BUTTONS1, r17
	rcall do_spi
	sts BUTTONS2, r17
		;------------------------
		;NEW
		ANALOG_DATA:
		rcall do_spi	;analog Data verwerfen
		rcall do_spi
		ANALOG_DATA_END:
		;------------------------
	SET_SS
	rcall translate_buttons
	rcall wait10ms
;read fifo status
RESET_SS
ldi r17, fifo_status
rcall do_spi
rcall do_spi
SET_SS
sbrs r17, 0
rjmp read_fifo
;IF FIFO=EMPTY THEN CLEAR RX_DR BIT
RESET_SS
ldi r17, write_reg+status
rcall do_spi
ldi r17, 0b01000000
rcall do_spi
SET_SS
SET_CE
rjmp main



;----------------------ISR0 - SATURN'S POLLING---------------------------
;------------------------------------------------------------------------
isr0:
		in r22, sreg ;save status register

		Sbic  Pind , Pd2
		rjmp H
		rjmp L
		H:
		sbic pind, pd3
		rjmp HH
		RJMP HL
		L:
		sbic pind, pd3
		rjmp LH
		RJMP LL
		Hh:
		;L 1 0 0
		Out Portc ,sathh
		rjmp isr0_end
		Lh:
		;R L D U
		Out Portc ,satlh
		rjmp isr0_end
		Hl:
		;START A C B
		Out Portc ,sathl
		rjmp isr0_end
		Ll:
		;R X Y Z
		Out Portc ,satll
isr0_end:
		out sreg, r22 ;write back status register
		reti
;---------------------ISR0 END-------------------------------------------
;------------------------------------------------------------------------



;------------------SETUP NRF----------------------------------------
;-------------------------------------------------------------------
;every new command must be started with a high to low transition of CSN (SS)
setup_nrf:
		SET_CE
		rcall wait10ms
		RESET_CE

		SET_SS
		rcall delay7
		RESET_SS
		ldi r17, write_reg+rx_addr_p0
		rcall do_spi
		ldi r17, 0x34
		rcall do_spi
		ldi r17, 0x43
		rcall do_spi
		ldi r17, 0x10
		rcall do_spi
		ldi r17, 0x10
		rcall do_spi
		ldi r17, 0x01
		rcall do_spi

		SET_SS
		rcall delay7
		RESET_SS
		ldi r17, write_reg+en_aa
		rcall do_spi
		ldi r17, 0x01
		rcall do_spi

		SET_SS
		rcall delay7
		RESET_SS
		ldi r17, write_reg+en_rxaddr
		rcall do_spi
		ldi r17, 0x01
		rcall do_spi

		SET_SS
		rcall delay7
		RESET_SS
		ldi r17, write_reg+rf_ch	;CHANNEL
		rcall do_spi
		LDS r17, CHANNEL ;LOAD CHANNEL
		;ldi r17 , 50
		rcall do_spi

		SET_SS
		rcall delay7
		RESET_SS
		ldi r17, write_reg+rx_pw_p0 ;PAYLOAD WIDTH
		rcall do_spi
		ldi r17, 5	;OLD:3
		rcall do_spi

		SET_SS
		rcall delay7
		RESET_SS
		ldi r17, write_reg+rf_setup
		rcall do_spi
		ldi r17, 0b0000_1110	;2MPS datarate
		rcall do_spi

		SET_SS
		rcall delay7
		RESET_SS
		ldi r17, write_reg+config_nrf
		rcall do_spi
		ldi r17, 0x0F
		rcall do_spi

		SET_SS
		rcall wait10ms
		SET_CE
		ret
;-------------------SETUP NRF END------------------
;----------------------------------------------------



;-------------------SPI ROUTINE--------------------
;--------------------------------------------------
do_spi:
	out spdr, r17 ;write out(x) in SPI data register
	spi_not_complete:
	sbis spsr, spif ;if SPI interrupt flag isn't set, then wait
	rjmp spi_not_complete
	in r17, spdr
ret
;--------------------SPI ROUTINE END------------------
;-----------------------------------------------------



;-----------------TRANSLATE BUTTONS------------------------------------------
;----------------------------------------------------------------------------
translate_buttons:
;<-------------------------- BYTE 1------------------------->   <---------------------BYTE 2---------------------------->
;UP    DOWN    LEFT    RIGHT    SELECT   START    A      B   |||   C      X      Y      Z      L1      L2      R1      R2
;7       6       5       4        3        2      1      0   |||   7      6      5      4       3       2       1       0
		;LOAD BUTTONS2
		lds r17, BUTTONS1
		;SAT LH
		ldi r16, 0b0001_1111
		;UP
		sbrs r17, 7
		cbr r16, 0b0000_0001
		;DOWN
		sbrs r17, 6
		cbr r16, 0b0000_0010
		;LEFT
		sbrs r17, 5
		cbr r16, 0b0000_0100
		;RIGHT
		sbrs r17, 4
		cbr r16, 0b0000_1000
		mov satlh, r16

		;SAT HL
		ldi r16, 0b0001_1111
		;START
		sbrs r17, 2
		cbr r16, 0b0000_1000
		;A
		sbrs r17, 1
		cbr r16, 0b0000_0100
		;B
		sbrs r17, 0
		cbr r16, 0b0000_0001
		;LOAD BUTTONS2
		lds r17, buttons2
		;C
		sbrs r17, 7
		cbr r16, 0b0000_0010
		mov sathl, r16

		;SAT LL
		ldi r16, 0b0001_1111
		;X
		sbrs r17, 6
		cbr r16, 0b0000_0100
		;Y
		sbrs r17, 5
		cbr r16, 0b0000_0010
		;Z
		sbrs r17, 4
		cbr r16, 0b0000_0001
		;R1
		sbrs r17, 1
		cbr r16, 0b0000_1000
		mov satll, r16

		;SAT HH
		ldi r16, 0b0001_1100
		sbrs r17, 3
		cbr r16, 0b0000_1000
		mov sathh, r16

		ret
;----------------TRANSLATE BUTTONS END------------------------
;-------------------------------------------------------------



;------------------CHANNEL_SELECT-------------------------------
;---------------------------------------------------------------
channel_select:
		ldi r16, 109
		sbis pind, channel_switch
		ldi r16, 119
		sts channel, r16
		ret
;-----------------CHANNEL_SELECT END----------------------------
;---------------------------------------------------------------



;-----------WAIT SUBROUTINES_----------------------------------
;--------------------------------------------------------------
delay7:
	ret
delay14:
	rcall delay7
	ret
delay16:
	rcall delay7
	nop
	nop
	ret
delay62:
	rcall delay16
	rcall delay16
	rcall delay16
	rcall delay7
	ret
wait10ms:
	ldi r16, 10
	wait10times:
		ldi r31, high(4000)
		ldi r30, low(4000)
		msloop:
		sbiw r30, 1
		brne msloop
	dec r16
	brne wait10times
	ret	
;---------WAIT ROUTINES END------------------------------------------
;------------------------------------------------------------------	







